@@ -1,6 +1,6 @@
 ## for now, this needs to be changed in setup.py also until I find a
 ## better solution
-__version__ = '0.9.2'
+__version__ = '0.10.0'
 __author__ = 'meejah'
 __contact__ = 'meejah@meejah.ca'
 __url__ = 'https://github.com/meejah/txtorcon'
@@ -13,9 +13,16 @@ from txtorcon.circuit import Circuit
 from txtorcon.stream import Stream
 from txtorcon.torcontrolprotocol import TorControlProtocol, TorProtocolError, TorProtocolFactory, DEFAULT_VALUE
 from txtorcon.torstate import TorState, build_tor_connection, build_local_tor_connection
-from txtorcon.torconfig import TorConfig, HiddenService, TorProcessProtocol, TCPHiddenServiceEndpoint, launch_tor, TorNotFound
+from txtorcon.torconfig import TorConfig, HiddenService, TorProcessProtocol, launch_tor, TorNotFound
 from txtorcon.torinfo import TorInfo
 from txtorcon.addrmap import AddrMap
+from txtorcon.endpoints import TorOnionAddress
+from txtorcon.endpoints import TorOnionListeningPort
+from txtorcon.endpoints import TCPHiddenServiceEndpoint
+from txtorcon.endpoints import TCPHiddenServiceEndpointParser
+from txtorcon.endpoints import IHiddenService
+from txtorcon.endpoints import IProgressProvider
+from txtorcon.endpoints import get_global_tor
 import util
 import interface
 from txtorcon.interface import *
@@ -29,11 +36,15 @@ __all__ = ["Router",
            "build_tor_connection", "build_local_tor_connection", "launch_tor", "TorNotFound",
            "TorConfig", "HiddenService", "TorProcessProtocol",
            "TorInfo",
-           "TCPHiddenServiceEndpoint",
+           "TCPHiddenServiceEndpoint", "TCPHiddenServiceEndpointParser",
+           "IHiddenService", "IProgressProvider",
+           "TorOnionAddress", "TorOnionListeningPort",
+           "get_global_tor",
+
            "AddrMap",
            "util", "interface",
            "ITorControlProtocol",
            "IStreamListener", "IStreamAttacher", "StreamListenerMixin",
            "ICircuitContainer", "ICircuitListener", "CircuitListenerMixin",
-           "IRouterContainer", "IAddrListener"
+           "IRouterContainer", "IAddrListener", "IProgressProvider", "IHiddenService",
            ]
