@@ -418,7 +418,7 @@ class TorControlProtocol(LineOnlyReceiver):
         :return: a ``Deferred`` which callbacks with Tor's response
             (``OK`` or something like ``552 Unrecognized signal code "foo"``).
         """
-        if not nm in self.valid_signals:
+        if nm not in self.valid_signals:
             raise RuntimeError("Invalid signal " + nm)
         return self.queue_command('SIGNAL %s' % nm)
 
@@ -435,6 +435,11 @@ class TorControlProtocol(LineOnlyReceiver):
         argument, that is the text collected for the event from the
         tor control protocol.
 
+        .. note::
+            this is a low-level interface; if you want to follow
+            circuit or stream creation etc. see TorState and methods
+            like add_circuit_listener
+
         :Return: ``None``
 
         .. todo::
@@ -442,7 +447,7 @@ class TorControlProtocol(LineOnlyReceiver):
             show how to tie in Stem parsing if you want
         """
 
-        if not evt in self.valid_events.values():
+        if evt not in self.valid_events.values():
             try:
                 evt = self.valid_events[evt]
             except:
@@ -455,7 +460,7 @@ class TorControlProtocol(LineOnlyReceiver):
         return None
 
     def remove_event_listener(self, evt, cb):
-        if not evt in self.valid_events.values():
+        if evt not in self.valid_events.values():
             try:
                 evt = self.valid_events[evt]
             except:
@@ -783,7 +788,7 @@ class TorControlProtocol(LineOnlyReceiver):
         self.response = ''
         if self.code >= 200 and self.code < 300:
             if self.defer is None:
-                raise RuntimeError("Got a response, but didn't issue a command.")
+                raise RuntimeError('Got a response, but didn\'t issue a command: "%s"' % resp)
             if resp.endswith('\nOK'):
                 resp = resp[:-3]
             self.defer.callback(resp)
